@@ -60,7 +60,6 @@
 #include <math.h>
 #include <sys/stat.h>
 #include <string.h>
-#include <clutter-cairo.h>
 
 enum
 {
@@ -154,9 +153,6 @@ champlain_network_map_source_set_property (GObject *object,
 static void
 champlain_network_map_source_dispose (GObject *object)
 {
-  //ChamplainNetworkMapSource *source = CHAMPLAIN_NETWORK_MAP_SOURCE (object);
-  //ChamplainNetworkMapSourcePrivate *priv = source->priv;
-
   if (soup_session != NULL)
       soup_session_abort (soup_session);
 }
@@ -239,7 +235,7 @@ champlain_network_map_source_init (ChamplainNetworkMapSource *champlainMapSource
 
   champlainMapSource->priv = priv;
 
-  priv->proxy_uri = g_strdup ("");
+  priv->proxy_uri = NULL;
   priv->uri_format = NULL;
   priv->offline = FALSE;
 }
@@ -256,7 +252,7 @@ champlain_network_map_source_init (ChamplainNetworkMapSource *champlainMapSource
  * @projection: the map source's projection
  * @uri_format: the URI to fetch the tiles from, see #champlain_network_map_source_set_uri_format
  *
- * Returns a constructed #ChamplainNetworkMapSource
+ * Returns: a constructed #ChamplainNetworkMapSource
  *
  * Since: 0.4
  */
@@ -287,7 +283,7 @@ champlain_network_map_source_new_full (const gchar *id,
  * @y: the y position of the tile
  * @z: the zool level of the tile
  *
- * Returns a contruscted URI with the given parameters based on the map
+ * Returns: a contruscted URI with the given parameters based on the map
  * source's URI format.
  *
  * Since: 0.4
@@ -332,6 +328,7 @@ champlain_network_map_source_get_tile_uri (ChamplainNetworkMapSource *source,
 
       token = tokens[++i];
     }
+
   token = ret->str;
   g_string_free (ret, FALSE);
   g_strfreev (tokens);
@@ -352,7 +349,6 @@ champlain_network_map_source_get_tile_uri (ChamplainNetworkMapSource *source,
  * For example, this is the OpenStreetMap URI format:
  * "http://tile.openstreetmap.org/#Z#/#X#/#Y#.png"
  *
- *
  * Since: 0.4
  */
 void
@@ -369,7 +365,6 @@ static gchar *
 get_filename (ChamplainNetworkMapSource *source,
     ChamplainTile *tile)
 {
-  //ChamplainNetworkMapSourcePrivate *priv = source->priv;
   return g_strdup_printf ("%s" G_DIR_SEPARATOR_S "%s" G_DIR_SEPARATOR_S
              "%s" G_DIR_SEPARATOR_S "%d" G_DIR_SEPARATOR_S
              "%d" G_DIR_SEPARATOR_S "%d.png", g_get_user_cache_dir (),
@@ -387,8 +382,8 @@ create_error_tile (ChamplainTile* tile)
   guint size;
 
   size = champlain_tile_get_size (tile);
-  actor = clutter_cairo_new (size, size);
-  cr = clutter_cairo_create (CLUTTER_CAIRO(actor));
+  actor = clutter_cairo_texture_new (size, size);
+  cr = clutter_cairo_texture_create (CLUTTER_CAIRO_TEXTURE (actor));
 
   /* draw a linear gray to white pattern */
   pat = cairo_pattern_create_linear (size / 2.0, 0.0,  size, size / 2.0);
@@ -450,20 +445,19 @@ file_loaded_cb (SoupSession *session,
     {
       /* Since we are updating the cache, we can assume that the directories
        * exists */
-      GTimeVal *now = g_new0 (GTimeVal, 1);
+      GTimeVal now = {0, };
 
       file = g_file_new_for_path (filename);
       info = g_file_query_info (file, G_FILE_ATTRIBUTE_TIME_MODIFIED,
           G_FILE_QUERY_INFO_NONE, NULL, NULL);
 
-      g_get_current_time (now);
-      g_file_info_set_modification_time (info, now);
+      g_get_current_time (&now);
+      g_file_info_set_modification_time (info, &now);
       g_file_set_attributes_from_info (file, info, G_FILE_QUERY_INFO_NONE, NULL,
           NULL);
 
       g_object_unref (file);
       g_object_unref (info);
-      g_free (now);
 
       champlain_tile_set_state (tile, CHAMPLAIN_STATE_DONE);
       g_object_unref (tile);
@@ -493,11 +487,10 @@ file_loaded_cb (SoupSession *session,
         {
           g_warning ("Unable to load the pixbuf: %s", error->message);
           g_error_free (error);
-          create_error_tile (tile);
-          goto cleanup;
         }
 
-      g_object_unref (loader);
+      create_error_tile (tile);
+      goto cleanup;
     }
 
   gdk_pixbuf_loader_close (loader, &error);
@@ -534,14 +527,13 @@ file_loaded_cb (SoupSession *session,
   DEBUG ("Received ETag %s", etag);
 
   if (etag != NULL)
-    {
-      champlain_tile_set_etag (tile, etag);
-    }
-  champlain_cache_update_tile (cache, tile, filesize); //XXX
+    champlain_tile_set_etag (tile, etag);
+
+  champlain_cache_update_tile (cache, tile, filesize);
 
   /* Load the image into clutter */
-  GdkPixbuf* pixbuf = gdk_pixbuf_loader_get_pixbuf(loader);
-  actor = clutter_texture_new();
+  GdkPixbuf* pixbuf = gdk_pixbuf_loader_get_pixbuf (loader);
+  actor = clutter_texture_new ();
   if (!clutter_texture_set_from_rgb_data (CLUTTER_TEXTURE (actor),
       gdk_pixbuf_get_pixels (pixbuf),
       gdk_pixbuf_get_has_alpha (pixbuf),
@@ -620,14 +612,14 @@ fill_tile (ChamplainMapSource *map_source,
       if (!soup_session)
         {
           soup_session = soup_session_async_new_with_options ("proxy-uri",
-                soup_uri_new (priv->proxy_uri),
+              soup_uri_new (priv->proxy_uri),
 #ifdef HAVE_LIBSOUP_GNOME
               SOUP_SESSION_ADD_FEATURE_BY_TYPE, SOUP_TYPE_PROXY_RESOLVER_GNOME,
 #endif
               NULL);
           g_object_set (G_OBJECT (soup_session),
               "user-agent", "libchamplain/" CHAMPLAIN_VERSION_S,
-              "max-conns-per-host", 8, NULL); // This is the same has Firefox
+              "max-conns-per-host", 2, NULL); // This is as required by OSM
           g_object_add_weak_pointer (G_OBJECT (soup_session),
               (gpointer *) &soup_session);
         }
@@ -680,10 +672,3 @@ fill_tile (ChamplainMapSource *map_source,
    */
   g_object_unref (cache);
 }
-
-void
-champlain_network_map_source_fill_tile (ChamplainMapSource *map_source,
-    ChamplainTile *tile)
-{
-  fill_tile (map_source, tile);
-}
