@@ -134,6 +134,12 @@ struct _ChamplainViewPrivate
   gdouble longitude;
   gdouble latitude;
 
+  /* Hack to get smaller x,y coordinates as the clutter limit is G_MAXINT16 */
+  ChamplainFloatPoint anchor;
+
+  gdouble anchor_zoom_level; /* the zoom_level for which the current anchor has
+                                been computed for */
+
   Map *map; /* Contains the current map model */
 
   ClutterActor *finger_scroll; /* Contains the viewport */
@@ -197,7 +203,10 @@ static gboolean finger_scroll_button_press_cb (ClutterActor *actor,
 static void update_license (ChamplainView *view);
 static void license_set_position (ChamplainView *view);
 static void view_load_visible_tiles (ChamplainView *view);
+static void view_position_tile (ChamplainView* view, ChamplainTile* tile);
+static void view_tiles_reposition (ChamplainView* view);
 static void view_update_state (ChamplainView *view);
+static void view_update_anchor (ChamplainView *view, gint x, gint y);
 static gboolean view_set_zoom_level_at (ChamplainView *view,
     gint zoom_level,
     gint x,
@@ -230,7 +239,7 @@ viewport_get_current_longitude (ChamplainViewPrivate *priv)
   if (!priv->map)
     return 0.0;
 
-  return viewport_get_longitude_at (priv,
+  return viewport_get_longitude_at (priv, priv->anchor.x +
       priv->viewport_size.x + priv->viewport_size.width / 2.0);
 }
 
@@ -251,7 +260,8 @@ viewport_get_current_latitude (ChamplainViewPrivate *priv)
     return 0.0;
 
   return viewport_get_latitude_at (priv,
-      priv->viewport_size.y + priv->viewport_size.height / 2.0);
+      priv->anchor.y + priv->viewport_size.y +
+      priv->viewport_size.height / 2.0);
 }
 
 static gboolean
@@ -285,7 +295,9 @@ marker_reposition_cb (ChamplainMarker *marker,
       x = champlain_map_source_get_x (priv->map_source, priv->zoom_level, marker_priv->lon);
       y = champlain_map_source_get_y (priv->map_source, priv->zoom_level, marker_priv->lat);
 
-      clutter_actor_set_position (CLUTTER_ACTOR (marker), x, y);
+      clutter_actor_set_position (CLUTTER_ACTOR (marker),
+        x - priv->anchor.x,
+        y - priv->anchor.y);
     }
 }
 
@@ -346,6 +358,7 @@ create_initial_map (ChamplainView *view)
   clutter_container_add_actor (CLUTTER_CONTAINER (priv->map_layer), group);
 
   g_idle_add (marker_reposition, view);
+  view_tiles_reposition (view);
   update_license (view);
 
   g_object_notify (G_OBJECT (view), "zoom-level");
@@ -389,15 +402,15 @@ draw_polygon (ChamplainView *view, ChamplainPolygon *polygon)
   while (list != NULL)
     {
       ChamplainPoint *point = (ChamplainPoint*) list->data;
-      gint x, y;
+      gfloat x, y;
 
       x = champlain_map_source_get_x (priv->map_source, priv->zoom_level,
           point->lon);
       y = champlain_map_source_get_y (priv->map_source, priv->zoom_level,
           point->lat);
 
-      x -= priv->viewport_size.x;
-      y -= priv->viewport_size.y;
+      x -= priv->viewport_size.x + priv->anchor.x;
+      y -= priv->viewport_size.y + priv->anchor.y;
 
       cairo_line_to (cr, x, y);
       list = list->next;
@@ -474,17 +487,33 @@ resize_viewport (ChamplainView *view)
   tidy_scrollable_get_adjustments (TIDY_SCROLLABLE (priv->viewport), &hadjust,
       &vadjust);
 
-  lower = -priv->viewport_size.width / 2.0;
-  upper = champlain_zoom_level_get_width (priv->map->current_level) *
-      champlain_map_source_get_tile_size (priv->map_source) -
-      priv->viewport_size.width / 2.0;
+  if (priv->zoom_level < 8)
+    {
+      lower = -priv->viewport_size.width / 2.0;
+      upper = champlain_zoom_level_get_width (priv->map->current_level) *
+          champlain_map_source_get_tile_size (priv->map_source) -
+          priv->viewport_size.width / 2.0;
+    }
+  else
+    {
+      lower = 0;
+      upper = G_MAXINT16;
+    }
   g_object_set (hadjust, "lower", lower, "upper", upper,
       "page-size", 1.0, "step-increment", 1.0, "elastic", TRUE, NULL);
 
-  lower = -priv->viewport_size.height / 2.0;
-  upper = champlain_zoom_level_get_height (priv->map->current_level) *
-      champlain_map_source_get_tile_size (priv->map_source) -
-      priv->viewport_size.height / 2.0;
+  if (priv->zoom_level < 8)
+    {
+      lower = -priv->viewport_size.height / 2.0;
+      upper = champlain_zoom_level_get_height (priv->map->current_level) *
+          champlain_map_source_get_tile_size (priv->map_source) -
+          priv->viewport_size.height / 2.0;
+    }
+  else
+    {
+      lower = 0;
+      upper = G_MAXINT16;
+    }
   g_object_set (vadjust, "lower", lower, "upper", upper,
       "page-size", 1.0, "step-increment", 1.0, "elastic", TRUE, NULL);
 
@@ -894,7 +923,6 @@ champlain_view_class_init (ChamplainViewClass *champlainViewClass)
 
   /**
   * ChamplainView::animation-completed:
-  * @view: the #ChamplainView that received the signal
   *
   * The ::animation-completed signal is emitted when any animation in the view
   * ends.  This is a detailed signal.  For example, if you want to be signaled
@@ -959,6 +987,9 @@ champlain_view_init (ChamplainView *view)
   priv->viewport_size.y = 0;
   priv->viewport_size.width = 0;
   priv->viewport_size.height = 0;
+  priv->anchor.x = 0;
+  priv->anchor.y = 0;
+  priv->anchor_zoom_level = 0;
   priv->state = CHAMPLAIN_STATE_INIT;
   priv->latitude = 0.0f;
   priv->longitude = 0.0f;
@@ -993,7 +1024,6 @@ champlain_view_init (ChamplainView *view)
   clutter_container_add_actor (CLUTTER_CONTAINER (priv->viewport),
       priv->map_layer);
 
-
   g_signal_connect (priv->finger_scroll, "button-press-event",
       G_CALLBACK (finger_scroll_button_press_cb), view);
   g_signal_connect_after (priv->finger_scroll, "button-release-event",
@@ -1037,6 +1067,7 @@ viewport_pos_changed_cb (GObject *gobject,
   ChamplainViewPrivate *priv = view->priv;
 
   ChamplainFloatPoint rect;
+  ChamplainFloatPoint old_anchor;
 
   tidy_viewport_get_origin (TIDY_VIEWPORT (priv->viewport), &rect.x, &rect.y,
       NULL);
@@ -1045,10 +1076,31 @@ viewport_pos_changed_cb (GObject *gobject,
       rect.y == priv->viewport_size.y)
       return;
 
+  old_anchor.x = priv->anchor.x;
+  old_anchor.y = priv->anchor.y;
+
+  view_update_anchor (view,
+      rect.x + priv->anchor.x + priv->viewport_size.width / 2.0,
+      rect.y + priv->anchor.y + priv->viewport_size.height / 2.0);
+
+  if (priv->anchor.x - old_anchor.x != 0)
+    {
+      ChamplainFloatPoint diff;
+
+      diff.x = priv->anchor.x - old_anchor.x;
+      diff.y = priv->anchor.y - old_anchor.y;
+
+      DEBUG("Relocating the viewport by %f, %f", diff.x, diff.y);
+      tidy_viewport_set_origin (TIDY_VIEWPORT (priv->viewport),
+          rect.x - diff.x, rect.y - diff.y, 0);
+      return;
+    }
+
   priv->viewport_size.x = rect.x;
   priv->viewport_size.y = rect.y;
 
   view_load_visible_tiles (view);
+  view_tiles_reposition (view);
   marker_reposition (view);
   view_update_polygons (view);
 
@@ -1059,6 +1111,18 @@ viewport_pos_changed_cb (GObject *gobject,
   g_object_notify (G_OBJECT (view), "latitude");
 }
 
+/**
+ * champlain_view_set_size:
+ * @view: a #ChamplainView
+ * @width: the width in pixels
+ * @height: the height in pixels
+ *
+ * Sets the size of the view.  This function will most probably be deprecated in
+ * future versions in favor of #clutter_actor_set_size.  In the mean time, you need
+ * to call both.
+ *
+ * Since: 0.1
+ */
 //FIXME: move to an handler of actor size change
 void
 champlain_view_set_size (ChamplainView *view,
@@ -1253,7 +1317,7 @@ finger_scroll_key_press_cb (ClutterActor *actor,
 /**
  * champlain_view_new:
  *
- * Returns a new #ChamplainView ready to be used as a #ClutterActor.
+ * Returns: a new #ChamplainView ready to be used as a #ClutterActor.
  *
  * Since: 0.4
  */
@@ -1263,6 +1327,58 @@ champlain_view_new (void)
   return g_object_new (CHAMPLAIN_TYPE_VIEW, NULL);
 }
 
+static void
+view_update_anchor (ChamplainView *view,
+    gint x,  /* Absolute x */
+    gint y)  /* Absolute y */
+{
+  ChamplainViewPrivate *priv = view->priv;
+  gboolean need_anchor = FALSE;
+  gboolean need_update = FALSE;
+
+  if (priv->zoom_level >= 8)
+    need_anchor = TRUE;
+
+  if (priv->anchor_zoom_level != priv->zoom_level ||
+      x - priv->anchor.x + priv->viewport_size.width >= G_MAXINT16 ||
+      y - priv->anchor.y + priv->viewport_size.height >= G_MAXINT16 ||
+      x - priv->anchor.x + priv->viewport_size.width <= 0 ||
+      y - priv->anchor.y + priv->viewport_size.height <= 0 )
+    need_update = TRUE;
+
+  if (need_anchor && need_update)
+    {
+      gdouble max;
+
+      priv->anchor.x = x - G_MAXINT16 / 2;
+      priv->anchor.y = y - G_MAXINT16 / 2;
+
+      if ( priv->anchor.x < 0 )
+        priv->anchor.x = 0;
+      if ( priv->anchor.y < 0 )
+        priv->anchor.y = 0;
+
+      max = champlain_zoom_level_get_width (priv->map->current_level) *
+          champlain_map_source_get_tile_size (priv->map_source) -
+          (G_MAXINT16 / 2);
+      if (priv->anchor.x > max)
+        priv->anchor.x = max;
+      if (priv->anchor.y > max)
+        priv->anchor.y = max;
+
+      priv->anchor_zoom_level = priv->zoom_level;
+      DEBUG ("New Anchor (%f, %f) at (%d, %d)", priv->anchor.x, priv->anchor.y, x, y);
+    }
+
+  if (need_anchor == FALSE)
+    {
+      priv->anchor.x = 0;
+      priv->anchor.y = 0;
+      priv->anchor_zoom_level = priv->zoom_level;
+      DEBUG ("Clear Anchor at (%d, %d)", x, y);
+    }
+}
+
 /**
  * champlain_view_center_on:
  * @view: a #ChamplainView
@@ -1294,6 +1410,13 @@ champlain_view_center_on (ChamplainView *view,
   x = champlain_map_source_get_x (priv->map_source, priv->zoom_level, longitude);
   y = champlain_map_source_get_y (priv->map_source, priv->zoom_level, latitude);
 
+  DEBUG ("Centering on %f, %f (%d, %d)", latitude, longitude, x, y);
+
+  view_update_anchor (view, x, y);
+
+  x -= priv->anchor.x;
+  y -= priv->anchor.y;
+
   tidy_viewport_set_origin (TIDY_VIEWPORT (priv->viewport),
     x - priv->viewport_size.width / 2.0,
     y - priv->viewport_size.height / 2.0,
@@ -1303,6 +1426,7 @@ champlain_view_center_on (ChamplainView *view,
   g_object_notify (G_OBJECT (view), "latitude");
 
   view_load_visible_tiles (view);
+  view_tiles_reposition (view);
   view_update_polygons (view);
   marker_reposition (view);
 }
@@ -1501,6 +1625,8 @@ champlain_view_set_zoom_level (ChamplainView *view,
   if (!map_zoom_to (priv->map, priv->map_source, zoom_level))
     return;
 
+  DEBUG ("Zooming to %d", zoom_level);
+
   priv->zoom_level = zoom_level;
   /* Fix to bug 575133: keep the lat,lon as it gets set to a wrong value
    * when resizing the viewport, when passing from zoom_level 7 to 6
@@ -1612,7 +1738,7 @@ champlain_view_add_layer (ChamplainView *view,
  * @lat: a variable where to put the latitude of the event
  * @lon: a variable where to put the longitude of the event
  *
- * Returns the latitude, longitude coordinates for the given ClutterEvent.
+ * Returns: the latitude, longitude coordinates for the given ClutterEvent.
  *
  * Since: 0.2.8
  */
@@ -1675,7 +1801,7 @@ champlain_view_get_coords_from_event (ChamplainView *view,
  * @lat: a variable where to put the latitude of the event
  * @lon: a variable where to put the longitude of the event
  *
- * Returns the latitude, longitude coordinates for the given x, y position in
+ * Returns: the latitude, longitude coordinates for the given x, y position in
  * the view.  Use if you get coordinates from GtkEvents for example.
  *
  * Since: 0.4
@@ -1698,10 +1824,10 @@ gboolean champlain_view_get_coords_at (ChamplainView *view,
 
   if (latitude)
     *latitude = viewport_get_latitude_at (priv,
-        priv->viewport_size.y + rel_y);
+        priv->viewport_size.y + rel_y + priv->anchor.y);
   if (longitude)
     *longitude = viewport_get_longitude_at (priv,
-        priv->viewport_size.x + rel_x);
+        priv->viewport_size.x + rel_x + priv->anchor.x);
 
   return TRUE;
 }
@@ -1714,6 +1840,9 @@ view_load_visible_tiles (ChamplainView *view)
   gint size;
   ChamplainZoomLevel *level;
 
+  viewport.x += priv->anchor.x;
+  viewport.y += priv->anchor.y;
+
   size = champlain_map_source_get_tile_size (priv->map_source);
   level = priv->map->current_level;
 
@@ -1764,7 +1893,8 @@ view_load_visible_tiles (ChamplainView *view)
           {
             actor = champlain_tile_get_actor (tile);
             group = champlain_zoom_level_get_actor (level);
-            clutter_container_remove_actor (CLUTTER_CONTAINER (group), actor);
+            if (actor != NULL)
+              clutter_container_remove_actor (CLUTTER_CONTAINER (group), actor);
           }
         champlain_zoom_level_remove_tile (level, tile);
         count = champlain_zoom_level_tile_count (level);
@@ -1797,8 +1927,7 @@ view_load_visible_tiles (ChamplainView *view)
             {
               DEBUG ("Loading tile %d, %d, %d", champlain_zoom_level_get_zoom_level (level), i, j);
               ChamplainTile *tile = champlain_tile_new ();
-              g_object_set (G_OBJECT (tile), "x", i, "y", j, "zoom-level",
-                  champlain_zoom_level_get_zoom_level (level), NULL);
+              g_object_set (G_OBJECT (tile), "x", i, "y", j, "zoom-level", champlain_zoom_level_get_zoom_level (level), NULL);
 
               g_signal_connect (tile, "notify::state", G_CALLBACK (tile_state_notify), view);
               clutter_container_add (CLUTTER_CONTAINER (champlain_zoom_level_get_actor (level)),
@@ -1814,11 +1943,55 @@ view_load_visible_tiles (ChamplainView *view)
   view_update_state (view);
 }
 
+static void
+view_position_tile (ChamplainView* view,
+    ChamplainTile* tile)
+{
+  ChamplainViewPrivate *priv = view->priv;
+
+  ClutterActor *actor;
+  gint x;
+  gint y;
+  guint size;
+
+  actor = champlain_tile_get_actor (tile);
+
+  if (actor == NULL)
+    return;
+
+  x = champlain_tile_get_x (tile);
+  y = champlain_tile_get_y (tile);
+  size = champlain_tile_get_size (tile);
+
+  clutter_actor_set_position (actor,
+    (x * size) - priv->anchor.x,
+    (y * size) - priv->anchor.y);
+}
+
+static void
+view_tiles_reposition (ChamplainView* view)
+{
+  ChamplainViewPrivate *priv = view->priv;
+  gint i;
+
+  for (i = 0; i < champlain_zoom_level_tile_count (priv->map->current_level); i++)
+    {
+      ChamplainTile *tile = champlain_zoom_level_get_nth_tile (priv->map->current_level, i);
+
+      if (tile == NULL)
+        continue;
+
+      if (champlain_tile_get_state (tile) == CHAMPLAIN_STATE_DONE)
+        view_position_tile (view, tile);
+    }
+}
+
 static void
 tile_state_notify (GObject *gobject,
     GParamSpec *pspec,
     gpointer data)
 {
+  view_position_tile (CHAMPLAIN_VIEW (data), CHAMPLAIN_TILE (gobject));
   view_update_state (CHAMPLAIN_VIEW (data));
 }
 
@@ -1906,6 +2079,7 @@ champlain_view_set_map_source (ChamplainView *view,
 
   update_license (view);
   g_idle_add (marker_reposition, view);
+  view_tiles_reposition (view);
   champlain_view_center_on (view, priv->latitude, priv->longitude);
 }
 
@@ -2186,9 +2360,9 @@ view_set_zoom_level_at (ChamplainView *view,
 
   /* Keep the lon, lat where the mouse is */
   lon = viewport_get_longitude_at (priv,
-    priv->viewport_size.x + rel_x);
+    priv->viewport_size.x + rel_x + priv->anchor.x);
   lat = viewport_get_latitude_at (priv,
-    priv->viewport_size.y + rel_y);
+    priv->viewport_size.y + rel_y + priv->anchor.y);
 
   /* How far was it from the center of the viewport (in px) */
   x_diff = priv->viewport_size.width / 2 - rel_x;
@@ -2225,7 +2399,7 @@ view_set_zoom_level_at (ChamplainView *view,
  * champlain_view_get_zoom_level:
  * @view: The view
  *
- * Returns the view's current zoom level.
+ * Returns: the view's current zoom level.
  *
  * Since: 0.4
  */
@@ -2243,7 +2417,7 @@ champlain_view_get_zoom_level (ChamplainView *view)
  * champlain_view_get_min_zoom_level:
  * @view: The view
  *
- * Returns the view's minimal zoom level allowed.
+ * Returns: the view's minimal zoom level allowed.
  *
  * Since: 0.4
  */
@@ -2261,7 +2435,7 @@ champlain_view_get_min_zoom_level (ChamplainView *view)
  * champlain_view_get_max_zoom_level:
  * @view: The view
  *
- * Returns the view's maximal zoom level allowed.
+ * Returns: the view's maximal zoom level allowed.
  *
  * Since: 0.4
  */
@@ -2279,7 +2453,7 @@ champlain_view_get_max_zoom_level (ChamplainView *view)
  * champlain_view_get_map_source:
  * @view: The view
  *
- * Returns the view's current map source. If you need to keep a reference to the
+ * Returns: the view's current map source. If you need to keep a reference to the
  * map source then you have to call #g_object_ref.
  *
  * Since: 0.4
@@ -2298,7 +2472,7 @@ champlain_view_get_map_source (ChamplainView *view)
  * champlain_view_get_decel_rate:
  * @view: The view
  *
- * Returns the view's deceleration rate.
+ * Returns: the view's deceleration rate.
  *
  * Since: 0.4
  */
@@ -2317,7 +2491,7 @@ champlain_view_get_decel_rate (ChamplainView *view)
  * champlain_view_get_scroll_mode:
  * @view: The view
  *
- * Returns the view's scroll mode behaviour.
+ * Returns: the view's scroll mode behaviour.
  *
  * Since: 0.4
  */
@@ -2334,7 +2508,7 @@ champlain_view_get_scroll_mode (ChamplainView *view)
  * champlain_view_get_keep_center_on_resize:
  * @view: The view
  *
- * Returns TRUE if the view keeps the center or resize.
+ * Returns: TRUE if the view keeps the center or resize, FALSE otherwise.
  *
  * Since: 0.4
  */
@@ -2351,7 +2525,7 @@ champlain_view_get_keep_center_on_resize (ChamplainView *view)
  * champlain_view_get_show_license:
  * @view: The view
  *
- * Returns TRUE if the view displays the license.
+ * Returns: TRUE if the view displays the license, FALSE otherwise.
  *
  * Since: 0.4
  */
@@ -2368,7 +2542,7 @@ champlain_view_get_show_license (ChamplainView *view)
  * champlain_view_get_zoom_on_double_click:
  * @view: The view
  *
- * Returns TRUE if the view zooms on double click.
+ * Returns: TRUE if the view zooms on double click, FALSE otherwise.
  *
  * Since: 0.4
  */
@@ -2386,7 +2560,7 @@ view_update_polygons (ChamplainView *view)
 {
   ChamplainViewPrivate *priv = view->priv;
   GList *polygons;
-  gint x, y;
+  gfloat x, y;
 
   if (priv->polygons == NULL)
     return;
